local cmp = require("cmp")
local luasnip = require("luasnip")

cmp.setup({
	formatting = {
		format = function(entry, vim_item)
			vim_item.menu = ({
				nvim_lsp = "[LSP]",
				buffer = "[Buffer]",
				luasnip = "[Snippet]",
				nvim_lua = "[API]",
				path = "[Path]",
				spell = "[Spell]",
			})[entry.source.name]
			vim_item.kind = ({
				Text = "  text",
				Method = "  Method",
				Function = "  Function",
				Constructor = "  Constructor",
				Field = "ﰠ  Field",
				Variable = "  Variable",
				Class = "  Class",
				Interface = "ﰮ  Interface",
				Module = "  Module",
				Property = "  Property",
				Unit = "  Unit",
				Value = "  Value",
				Enum = "  Enum",
				Keyword = "  Keyword",
				Snippet = "﬌  Snippet",
				Color = "  Color",
				File = "  File",
				Reference = "  Reference",
				Folder = "  Folder",
				EnumMember = "  EnumMember",
				Constant = "  Constant",
				Struct = "  Struct",
				Event = "  Event",
				Operator = "  Operator",
				TypeParameter = "  TypeParameter",
			})[vim_item.kind]
			return vim_item
		end,
	},
	mapping = {
		["<C-p>"] = cmp.mapping.select_prev_item(),
		["<C-n>"] = cmp.mapping.select_next_item(),
		["<C-d>"] = cmp.mapping.scroll_docs(-4),
		["<C-f>"] = cmp.mapping.scroll_docs(4),
		["<C-Space>"] = cmp.mapping(function(_)
			return vim.fn.pumvisible() == 1 and cmp.close() or cmp.complete()
		end),
		["<C-e>"] = cmp.mapping.close(),
		["<CR>"] = cmp.mapping.confirm({
			behavior = cmp.ConfirmBehavior.Replace,
			select = true,
		}),
		["<Tab>"] = function(fallback)
			if cmp.visible() then
				cmp.select_next_item()
			elseif luasnip.expand_or_jumpable() then
				vim.fn.feedkeys(vim.api.nvim_replace_termcodes("<Plug>luasnip-expand-or-jump", true, true, true), "")
			else
				fallback()
			end
		end,
		["<S-Tab>"] = function(fallback)
			if cmp.visible() then
				cmp.select_prev_item()
			elseif luasnip.jumpable(-1) then
				vim.fn.feedkeys(vim.api.nvim_replace_termcodes("<Plug>luasnip-jump-prev", true, true, true), "")
			else
				fallback()
			end
		end,
	},
	sources = cmp.config.sources({
		{ name = "luasnip", max_item_count = 5 },
		{ name = "nvim_lsp", max_item_count = 5 },
		{ name = "path", max_item_count = 10 },
		{ name = "buffer", max_item_count = 10 },
	}),
	snippet = {
		expand = function(args)
			require("luasnip").lsp_expand(args.body)
		end,
	},
})
